unit uDictClient;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, Menus, ExtCtrls, ComCtrls, IMyDictSrvc, cUnicodeCodecs;

const
  stInit:byte = 0;
  stAdd:byte = 1;
  stModify:byte = 2;

type

  TfrmExp = class(TForm)
    gbExpEdit: TGroupBox;
    mMenu: TMainMenu;
    File1: TMenuItem;
    Edit1: TMenuItem;
    NewGroup1: TMenuItem;
    Exit1: TMenuItem;
    sgExpressions: TStringGrid;
    cbxGroup: TComboBox;
    Label1: TLabel;
    edtExpression: TEdit;
    Label2: TLabel;
    cbxLanguage: TComboBox;
    Label3: TLabel;
    Panel1: TPanel;
    btnAdd: TButton;
    btnDelete: TButton;
    btnFilter: TButton;
    btnClearFilter: TButton;
    edtId: TEdit;
    Label4: TLabel;
    StatusBar1: TStatusBar;
    btnModify: TButton;
    btnSave: TButton;
    chgTimer: TTimer;
    btnCancel: TButton;
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure chgTimerTimer(Sender: TObject);
    procedure edtExpressionChange(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnModifyClick(Sender: TObject);
    procedure btnFilterClick(Sender: TObject);
    procedure btnClearFilterClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
  private
    { Private declarations }
    fState:byte;
    fMySrvc:IMyDictService;
    fgroups:TArrayOfGroup;
    fexpressions:TArrayOfExpression;
    fFiltered:boolean;
    procedure setState(iState:byte);
    procedure setButtons();
    procedure setEditorsEnabled();
    procedure clearEditors();
    procedure clearExpressions();
    function findIndex(gid:integer):integer;
    procedure getExpressions(gid:integer;lang:string);
    procedure setFiltered(value:boolean);
    procedure fillGrid;
  public
    { Public declarations }
    property state:byte read fstate write setState;
    property filtered:boolean read fFiltered write setFiltered;
  end;

var
  frmExp: TfrmExp;

implementation
{$R *.dfm}

procedure TfrmExp.clearExpressions();
var i:integer;
begin
  for i:=0 to length(fexpressions)-1 do
    if assigned(fexpressions[i]) then
      fexpressions[i].Free;
  setlength(fexpressions,0);
end;

procedure TfrmExp.Exit1Click(Sender: TObject);
begin
  close;
end;

procedure TfrmExp.FormCreate(Sender: TObject);
begin
  fMySrvc:=GetIMyDictService();
  fFiltered:=false;
end;

procedure TfrmExp.FormShow(Sender: TObject);
var i:integer;
begin
  fgroups:=fMySrvc.getGroups();
  cbxGroup.Items.Clear;
  cbxGroup.Items.Add('');
  sgExpressions.RowCount:=2;
  sgExpressions.Cells[0,0]:='id';
  sgExpressions.Cells[1,0]:='gid';
  sgExpressions.Cells[2,0]:='lang';
  sgExpressions.Cells[3,0]:='expression';
  for i:=0 to length(fgroups)-1 do
  begin
    cbxGroup.items.Add(UTF8StringToWideString(fgroups[i].group));
  end;
  getExpressions(-1,'');
  state:=stInit;
end;

procedure TfrmExp.chgTimerTimer(Sender: TObject);
var gid:integer;
    tmp:TExpression;
begin
  if (edtId.Text<>'') and (cbxLanguage.ItemIndex>0) then
  try
    tmp:=fmySrvc.getWordById(strToInt(edtId.Text),cbxLanguage.Items[cbxLanguage.ItemIndex]);
    clearExpressions;
    if assigned(tmp) then
    begin
      setLength(fexpressions,1);
      fexpressions[0]:=tmp;
    end;
    fillGrid;
  except
  end
  else if (cbxGroup.ItemIndex>-1) or (cbxLanguage.ItemIndex>-1) then
  begin
    if cbxGroup.ItemIndex>0 then
      gid:=fgroups[cbxGroup.ItemIndex-1].id
    else
      gid:=-1;
    getExpressions(gid,cbxLanguage.Items[cbxLanguage.ItemIndex]);
  end;
  chgTimer.Enabled:=false;
end;

procedure TfrmExp.edtExpressionChange(Sender: TObject);
begin
  chgTimer.Enabled:=filtered;
end;

procedure TfrmExp.btnAddClick(Sender: TObject);
begin
  State:=stAdd;
end;

procedure TfrmExp.btnSaveClick(Sender: TObject);
var tmpStr:WideString;
    result:TExpression;
    id:integer;
begin
  tmpStr:=WideStringToUTF8String(edtExpression.Text);
  if state=stAdd then
  begin
    try
      id:=strToInt(edtId.Text);
    except
      id:=-1;
    end;
    result:=fmySrvc.addExpression(id, cbxLanguage.Items[cbxLanguage.ItemIndex],fgroups[cbxGroup.ItemIndex-1].id,tmpStr);
  end
  else
    result:=fmySrvc.modifyExpression(fexpressions[sgExpressions.Row-1].id, trim(cbxLanguage.Text), fgroups[cbxGroup.ItemIndex-1].id,WideStringToUTF8String(edtExpression.Text));
  if (result<>nil) then
  begin
    ShowMessage('Saved (id:'+intToStr(result.id)+', lang:'+result.lang+')');
    getExpressions(-1,'');
    state:=stInit;
  end
  else
    ShowMessage('Expression not saved!');
end;

procedure TfrmExp.setState(iState: byte);
begin
  fState:=iState;
  if fState=stInit then
    clearEditors();
  setButtons();
  setEditorsEnabled();
end;

procedure TfrmExp.setButtons;
begin
  btnSave.Enabled:=(state in [stAdd,stModify]) and not filtered;
  btnCancel.Enabled:=btnSave.Enabled;
  btnAdd.Enabled:=(state in [stInit]) and not filtered;
  btnModify.Enabled:=(state in [stInit]) and (length(fexpressions)>0) and not filtered;
  btnDelete.Enabled:=btnModify.Enabled and not filtered;
  btnFilter.Enabled:=not (state in [stAdd,stModify]) and not filtered;
  btnClearFilter.Enabled:=not (state in [stAdd,stModify]) and filtered;
end;

procedure TfrmExp.btnModifyClick(Sender: TObject);
begin
  state:=stModify;
  setEditorsEnabled();
  edtId.Text:=intToStr(fexpressions[sgExpressions.Row-1].id);
  cbxGroup.ItemIndex:=findIndex(fexpressions[sgExpressions.Row-1].gid)+1;
  cbxLanguage.ItemIndex:=cbxLanguage.Items.IndexOf(trim(fexpressions[sgExpressions.Row-1].lang));
  edtExpression.Text:=UTF8StringToWideString(fexpressions[sgExpressions.Row-1].value);
end;

procedure TfrmExp.setEditorsEnabled;
var i:integer;
  tmp:boolean;
begin
  tmp:=(state in [stAdd,stModify]) or filtered;
  for i:=0 to gbExpEdit.ControlCount-1 do
    gbExpEdit.Controls[i].Enabled:=tmp;
  if not Filtered then
  begin
    edtId.Enabled:=(state=stAdd);
    cbxLanguage.Enabled:=edtId.Enabled;
  end;
end;

procedure TfrmExp.btnFilterClick(Sender: TObject);
begin
  Filtered:=true;
  setEditorsEnabled;
  setButtons;
end;

procedure TfrmExp.btnClearFilterClick(Sender: TObject);
begin
  Filtered:=false;
  clearEditors;
  setEditorsEnabled;
  setButtons;
  getExpressions(-1,'');
end;

procedure TfrmExp.getExpressions(gid:integer;lang:string);
begin
  clearExpressions;
  fexpressions:=fMySrvc.getWordsByGroupId(gid,lang);
  fillGrid;
end;

procedure TfrmExp.btnCancelClick(Sender: TObject);
begin
  state:=stInit;
  //clearEditors;
end;

procedure TfrmExp.clearEditors;
begin
  edtId.Text:='';
  edtExpression.Text:='';
  cbxGroup.ItemIndex:=0;
  cbxLanguage.ItemIndex:=0;
end;

procedure TfrmExp.btnDeleteClick(Sender: TObject);
begin
  if (MessageDlg('Are you soure?',mtConfirmation,mbOKCancel,0)=mrOK) and fMySrvc.deleteWord(fexpressions[sgExpressions.Row-1].id,trim(fexpressions[sgExpressions.Row-1].lang)) then
  begin
    ShowMessage('Row deleted.');
    getExpressions(-1,'');
  end;
end;

function TfrmExp.findIndex(gid: integer): integer;
var i:integer;
begin
  result:=-1;
  for i:=0 to length(fgroups) do
    if fgroups[i].id = gid then
    begin
      result:=i;
      break;
    end;
end;

procedure TfrmExp.setFiltered(value: boolean);
begin
  fFiltered:=value;
  btnFilter.Enabled:=not value;
  btnClearFilter.Enabled:=value;
end;

procedure TfrmExp.fillGrid;
var i:integer;
begin
  sgExpressions.RowCount:=2;
  for i:=0 to length(fexpressions)-1 do
  if assigned(fexpressions[i]) then
    begin
      sgExpressions.Cells[0,sgExpressions.RowCount-1]:=intToStr(fexpressions[i].id);
      sgExpressions.Cells[1,sgExpressions.RowCount-1]:=intToStr(fexpressions[i].gid);
      sgExpressions.Cells[2,sgExpressions.RowCount-1]:=fexpressions[i].lang;
      sgExpressions.Cells[3,sgExpressions.RowCount-1]:=UTF8StringToWideString(fexpressions[i].value);
      sgExpressions.RowCount:=sgExpressions.RowCount+1;
    end;
  if length(fexpressions)>0 then
    sgExpressions.RowCount:=sgExpressions.RowCount-1
  else
    for i:=0 to 3 do
      sgExpressions.Cells[i,1]:='';
end;

end.
