{ Invokable implementation File for TMyDictService which implements IMyDictService }

unit MyDictServiceImpl;

interface

uses InvokeRegistry, Types, XSBuiltIns, MyDictServiceIntf, DB, IBCustomDataSet, IBQuery, IBDatabase, SysUtils, Classes;

type

  { TMyDictService }
  TMyDictService = class(TInvokableClass, IMyDictService)
  private
    ibdbDict: TIBDatabase;
    ibqWork: TIBQuery;
    ibTrans: TIBTransaction;
    procedure createConn;
    procedure freeConn;
    procedure writeLog(ex:Exception);overload;
    procedure writeLog(msg:string);overload;
  public
    function addGroup(name:WideString):TGroup;stdcall;
    function modifyGroup(id:integer;name:WideString):TGroup;stdcall;
    function addExpression(id:integer;lang:string;gid:integer;expression:WideString):TExpression;stdcall;
    function modifyExpression(id:integer;lang:string;gid:integer;expression:WideString):TExpression;stdcall;
    function getGroups():TArrayOfGroup;stdcall;
    function getWordById(id:integer; lang: string):TExpression;stdcall;
    function getWordsByGroupId(gid:integer; lang: string):TArrayOfExpression;stdcall;
    function deleteWord(id:integer; lang: string):boolean;stdcall;
  end;

implementation
{ TMyDictService }

function TMyDictService.addExpression(id:integer; lang: string; gid: integer;
  expression: WideString): TExpression;
begin
  result:=nil;
  try
    createConn;
    try
      if id<0 then
      begin
        ibqWork.Active:=false;
        ibqWork.SQL.Text:='SELECT GEN_ID( gen_expressions_id, 1 ) FROM RDB$DATABASE;';
        ibqWork.Active:=true;
        id:=ibqWork.Fields[0].AsInteger;
      end;
      ibqWork.Active:=false;
      ibqWork.SQL.Text:='insert into expressions (id,gid,lang,expression) values (:ID,:GID,:LANG,:EXP)';
      ibqWork.ParamByName('ID').AsInteger:=id;
      ibqWork.ParamByName('GID').AsInteger:=gid;
      ibqWork.ParamByName('LANG').AsString:=lang;
      ibqWork.ParamByName('EXP').AsString:=expression;
      writeLog('new query: '+ibqWork.SQL.Text);
      ibqWork.ExecSQL;
      ibqWork.Active:=false;
      result:=TExpression.Create;
      result.id:=id;
      result.gid:=gid;
      result.lang:=lang;
      result.value:=expression;
    except
      on ex:Exception do
      begin
        writeLog(ex);
      end;
    end;
  finally
    freeConn;
  end;
end;

function TMyDictService.addGroup(name: WideString): TGroup;
var id:integer;
begin
  result:=nil;
  try
    createConn;
    try
      ibqWork.Active:=false;
      ibqWork.SQL.Text:='SELECT GEN_ID( gen_groups_id, 1 ) FROM RDB$DATABASE;';
      ibqWork.Active:=true;
      id:=ibqWork.Fields[0].AsInteger;
      ibqWork.Active:=false;
      ibqWork.SQL.Text:='insert into groups (id,groupname) values (:ID,:GNAME)';
      ibqWork.ParamByName('ID').AsInteger:=id;
      ibqWork.ParamByName('GNAME').AsString:=name;
      writeLog('new query: '+ibqWork.SQL.Text);
      ibqWork.ExecSQL;
      ibqWork.Active:=false;
      result:=TGroup.Create();
      result.id:=id;
      result.group:=name;
    except
      on ex:Exception do
      begin
        writeLog(ex);
      end;
    end;
  finally
    freeConn;
  end;
end;

procedure TMyDictService.createConn;
begin
  writeLog('create connection...');
  ibdbDict:=TIBDatabase.Create(nil);
  ibqWork:=TIBQuery.Create(nil);
  ibTrans:=TIBTransaction.Create(nil);

  ibdbDict.DatabaseName:='c:\develop\statlogics\DICT.GDB';
  ibdbDict.Params.Add('user_name=lx');
  ibdbDict.Params.Add('password=lx');
  ibdbDict.Params.Add('lc_ctype=UTF8');
  ibdbDict.LoginPrompt:=false;
  ibqWork.Database:=ibdbDict;

  ibTrans.DefaultDatabase:=ibdbDict;
  ibdbDict.DefaultTransaction:=ibTrans;
  ibqWork.Transaction:=ibTrans;
  ibdbDict.Connected:=true;
  writeLog('connected.');
end;

procedure TMyDictService.freeConn;
begin
  writeLog('close connection...');
  ibqWork.Active:=false;
  ibdbDict.Connected:=false;
  ibTrans.Free;
  ibqWork.Free;
  ibdbDict.Free;
  writeLog('disconnected.');
end;

function TMyDictService.getGroups(): TArrayOfGroup;
begin
  try
    createConn;
    try
      ibqWork.SQL.Text:='select * from groups';
      writeLog('new query: '+ibqWork.SQL.Text);
      ibqWork.Active:=true;
      ibqWork.First;
      while not ibqWork.Eof do
      begin
        setLength(result,ibqWork.RecordCount);
        result[ibqWork.RecordCount-1]:=TGroup.Create;
        result[ibqWork.RecordCount-1].id:=ibqWork.FieldByName('id').AsInteger;
        result[ibqWork.RecordCount-1].group:=ibqWork.FieldByName('GroupName').AsString;
        ibqWork.Next;
      end;
    except
      on ex:Exception do
        writeLog(ex);
    end;
  finally
    freeConn;
  end;
end;

function TMyDictService.getWordById(id: integer; lang: string): TExpression;
begin
  result:=nil;
  try
    createConn;
    try
      ibqWork.SQL.Text:='select expression,gid from expressions where id=:ID and lang=:LANG';
      writeLog('new query: '+ibqWork.SQL.Text);
      ibqWork.ParamByName('ID').Value:=id;
      ibqWork.ParamByName('LANG').Value:=lang;
      ibqWork.Active:=true;
      ibqWork.First;
      if not ibqWork.Eof then
      begin
        result:=TExpression.Create;
        result.id:=id;
        result.lang:=lang;
        result.gid:=ibqWork.FieldByName('gid').AsInteger;
        result.value:=ibqWork.FieldByName('expression').AsString;
      end;
    except
      on ex:Exception do
        writeLog(ex);
    end;
  finally
    freeConn;
  end;
end;

function TMyDictService.getWordsByGroupId(
  gid: integer; lang: string): TArrayOfExpression;
begin
  try
    createConn;
    try
      ibqWork.SQL.Text:='select * from expressions';
      if (gid>-1) or (lang<>'') then
        ibqWork.SQL.Text:=ibqWork.SQL.Text+' where ';

      if gid>-1 then
        ibqWork.SQL.Text:=ibqWork.SQL.Text+'gid=:GID';

      if lang<>'' then
      begin
        if gid>-1 then
          ibqWork.SQL.Text:=ibqWork.SQL.Text+' and ';
        ibqWork.SQL.Text:=ibqWork.SQL.Text+'lang=:LANG'
      end;

      if gid>-1 then
        ibqWork.ParamByName('GID').Value:=gid;
      if lang<>'' then
        ibqWork.ParamByName('LANG').Value:=lang;

      writeLog('new query: '+ibqWork.SQL.Text);
      ibqWork.Active:=true;
      ibqWork.First;
      while not ibqWork.Eof do
      begin
        setLength(result,ibqWork.RecordCount);
        result[ibqWork.RecordCount-1]:=TExpression.Create;
        result[ibqWork.RecordCount-1].id:=ibqWork.FieldByName('id').AsInteger;
        result[ibqWork.RecordCount-1].gid:=ibqWork.FieldByName('gid').AsInteger;
        result[ibqWork.RecordCount-1].value:=ibqWork.FieldByName('expression').AsString;
        result[ibqWork.RecordCount-1].lang:=ibqWork.FieldByName('lang').AsString;
        ibqWork.Next;
      end;
    except
      on ex:Exception do
        writeLog(ex);
    end;
  finally
    freeConn;
  end;
end;

procedure TMyDictService.writeLog(ex: Exception);
var f:TFileStream;
begin
  f:=TFileStream.Create('c:\temp\cgi.log',fmCreate);
  f.Seek(0,soFromEnd);
  f.Write(pchar(ex.Message+#13#10)^,length(ex.Message));
  f.Destroy;
end;

function TMyDictService.modifyExpression(id:integer;lang:string;gid:integer;expression:WideString): TExpression;
begin
  result:=nil;
  try
    createConn;
    try
      ibqWork.Active:=false;
      ibqWork.SQL.Text:='update expressions set gid=:GID,expression=:EXP where id=:ID and lang=:LANG';
      ibqWork.ParamByName('ID').AsInteger:=id;
      ibqWork.ParamByName('GID').AsInteger:=gid;
      ibqWork.ParamByName('LANG').AsString:=lang;
      ibqWork.ParamByName('EXP').AsString:=expression;
      writeLog('new query: '+ibqWork.SQL.Text);
      ibqWork.ExecSQL;
      if ibqWork.RowsAffected>0 then
      begin
        result:=TExpression.Create;
        result.id:=id;
        result.gid:=gid;
        result.lang:=lang;
        result.value:=expression;
      end;
      ibqWork.Active:=false;
    except
      on ex:Exception do
      begin
        writeLog(ex);
      end;
    end;
  finally
    freeConn;
  end;
end;

function TMyDictService.modifyGroup(id: integer; name: WideString): TGroup;
begin
  result:=nil;
end;

procedure TMyDictService.writeLog(msg:string);
var f:TFileStream;
begin
  f:=TFileStream.Create('c:\temp\cgi.log',fmCreate);
  f.Seek(0,soFromEnd);
  f.Write(pchar(msg+#13#10)^,length(msg));
  f.Destroy;
end;

function TMyDictService.deleteWord(id: integer; lang: string): boolean;
begin
  result:=false;
  try
    createConn;
    try
      ibqWork.SQL.Text:='delete from expressions where id=:ID and lang=:LANG';
      writeLog('new query: '+ibqWork.SQL.Text);
      ibqWork.ParamByName('ID').Value:=id;
      ibqWork.ParamByName('LANG').Value:=lang;
      ibqWork.ExecSQL;
      result:=ibqWork.RowsAffected>0;
    except
      on ex:Exception do
        writeLog(ex);
    end;
  finally
    freeConn;
  end;
end;

initialization
  { Invokable classes must be registered }
  InvRegistry.RegisterInvokableClass(TMyDictService);

end.
